package atome;
import simu.*;
import outils.*;

/**
 * Classe représentant un atome unique. Permet de suivre l'évolution de sa position et sa vitesse sous l'action des laser sans champ magnétique.
 *
 *
 *
 */

public class AtomeUnique extends SimuApplet {

    static double c=299792458.,mu=9.27400949e-24;
    double b,t,v0,sigma,deltat,lambdal;
    int i;
    double tau=0.,lambda0=0.,mass=0.;
    String atome;


    public AtomeUnique() {

    appletDoc("AtomeUnique_doc.xml");

    }


    public ListeValeurs calcul(ListeValeurs in) throws SimuException {

        deltat = in.lireDouble("deltat");
        v0 = in.lireDouble("v0");                //lecture des données utilisateur
        atome = in.lireString("atome");
        if( "Rubidium".equals(atome) ) {
            lambda0=780.;
            tau=27e-9;
            mass=85.*1.67492728e-27;
        }


        double vt=v0,vt_mem=2*v0,ft,err,dt,tol,Ya=0.,Yb=0.,t_mem,xt=0,xt_mem=0;
        double w0=2.*Math.PI*c/(lambda0*1e-9);
        double wl=w0*(1 + in.lireDouble("dw") );
        ListeValeurs out = new ListeValeurs();

        Force force = new Force(wl,w0,tau,mass);
        dt=0.1;
        tol=1e-8;
        t=0;
        ft=force.y(vt);
        out.ajouterDouble("t", t);
        out.ajouterDouble("vt", vt);
        out.ajouterDouble("ft", ft);
        out.ajouterDouble("xt", xt);

        while (t<deltat ) {
            vt_mem=vt;
            t_mem=t;
            xt_mem=xt;
            err=2.;

            dt = Bibliotheque.rk4_pasVariable(force,vt_mem,dt,tol); // on trouve le pas de temps adapté avec
                                                                    // rk4_pasVariable
            vt = Bibliotheque.rk4(force,vt_mem,dt); // rk4 avec le bon pas de temps
            t = t_mem+dt;
            xt = xt_mem+vt*dt;
            ft=Math.abs(force.y(vt));

            out.ajouterDouble("t", t);
            out.ajouterDouble("vt", vt);
            out.ajouterDouble("ft", ft);
            out.ajouterDouble("xt", xt);
        }

        for (i=-500;i<501;i++){

            ft=force.y(i);
            out.ajouterDouble("f",ft);
            out.ajouterDouble("v",i*1.);
        }

        return out;
    }
 }
