package population;
import java.util.Random;
import simu.*;
import outils.*;

/**
 * Population d'atomes
 * Permet de créer une distribution gaussienne des vitesses des atomes
 * et de la faire évoluer
 */

public class Population {

    int Natome;


/**
 *
 *Constructeur de l'objet Population. prend en argument le nombre d'atomes dans notre expérience.
 *
 *@param Natome Nombre d'individus dans notre population d'atomes
 *
 *
 */


    public Population(int Natome) {

        this.Natome=Natome;
    }




/**
 *
 *Méthode de l'objet Population, permettant d'initialiser la distribution des vitesses selon une gaussienne d'écart-type et de vitesse moyenne choisis par l'utilisateur.
 *
 *@param sigma écart-type de la gaussienne
 *@param vo vitesse moyenne de la gaussienne
 *
 */


    public double[]  InitV(double sigma, double vo){

        Random gauss = new Random();
        double[] vitesses = new double[Natome];
        for (int i=0;i<Natome;i++){
            vitesses[i]=vo + gauss.nextGaussian()*sigma;
        }

        return vitesses;

    }


/**
 *
 *Méthode de l'objet Population, permettant d'initialiser la distribution des positions selon une gaussienne d'écart-type choisi et de position moyenne nulle.
 *
 *@param sigma écart-type de la gaussienne
 *
 */


    public double[]  InitX(double sigma){

        Random gauss = new Random();
        double[] positions = new double[Natome];
        for (int i=0;i<Natome;i++){
            positions[i]= gauss.nextGaussian()*sigma;
        }

        return positions;
    }


/**
 *
 *Méthode de l'objet population, permettant de suivre l'évolution de la population d'atomes sous l'action des lasers.
 *
 *@param deltat durée de l'action des lasers
 *@param vitesses[] tableau contenant les vitesses initiales de tous les atomes de la population
 *@param positions[] tableau contenant les positions initiales de tous les atomes de la population
 *@param force force exercée par les lasers sur les atomes
 *
 */

public double[] Evolution(double deltat, double vitesses[], double positions[], Force force) throws SimuException {

    double[] vt_mem = new double[Natome];
    double[] vt = new double[Natome];
    double[] xt_mem = new double[Natome];
    double[] xt = new double[Natome];
    double[] SortiePositionVitesse = new double[2*Natome];
    double t=0.0, t_mem=0.0, tol=1.0e-08, err=2.0, dt=0.1, Ya=0.0, Yb=0.0;

 // Initialisations

    for (int i=0; i<Natome; i++) {
        vt_mem[i] = 2.0*vitesses[i];
        vt[i] = vitesses[i];
        xt[i] = positions[i];
    }

 // Boucle

    for(int i=0; i<Natome; i++) {
        t=0.0;
        t_mem=0.0;
        dt=0.1;

        while ( (vt_mem[i]-vt[i])/vitesses[i] > 0.00001 && t<deltat ) {
            vt_mem[i]=vt[i];
            xt_mem[i]=xt[i];
            t_mem=t;

            dt = Bibliotheque.rk4_pasVariable(force,vt_mem[i],dt,tol); // on trouve le pas de temps adapté avec
                                                                       // rk4_pasVariable
            vt[i] = Bibliotheque.rk4(force,vt_mem[i],dt); // rk4 avec le bon pas de temps
            t = t_mem+dt;
            xt[i] = xt_mem[i]+vt[i]*dt;
        }

        SortiePositionVitesse[i] = xt[i];
        SortiePositionVitesse[Natome+i] = vt[i];
    }

    return SortiePositionVitesse;
}



/**
 *
 *Méthode de l'objet Population, qui permet de faire les histogrammes de la distribution de vitesse, Avant ou après l'action des lasers
 *
 *@param vitesses tableau contenant les vitesses de tous les atomes de la population
 *@param Nhisto nombre d'histogrammes voulus
 *@param a borne inférieure de l'affichage des histogrammes
 *@param h largeur ou résolution des histogrammes
 *
 */

    public double[] Histo( double[] vitesses, int Nhisto, double a, double h){

        double[] histo = new double[Nhisto];

        for (int i=0;i<Nhisto;i++){
            histo[i]=0;
        }


        for (int i=0;i<Nhisto;i++){
            for(int j=0;j<Natome;j++){
                if(vitesses[j] > a+i*h && vitesses[j] < a+(i+1)*h){
                    histo[i]++;
                }
            }
        }
        return histo;
    }
}