package population;
import simu.*;
import outils.*;

/**
 * Classe simulant une population d'atomes soumis à la force de freinage laser.
 *
 *
 *
 */

public class PopulationAtomes extends SimuApplet {


    static double c=299792458.;
    double t,v0,sigma,lambdal,desaccordRelatif,xmin,xmax;
    int i;
    int Nhistos=100,Natomes;
    double[] Vatomes,VatomesInit,Xatomes,XatomesInit,VariablesAtomes;
    double[] Histogramme = new double[Nhistos];
    double[] FonctionRepartition = new double[Nhistos];
    Population Pop;
    double tau=0.,lambda0=0.,mass=0.;
    String atome,debut,fin;
    Affichage Aff;

    public PopulationAtomes() {

        appletDoc("PopulationAtomes_doc.xml");
    }

    public void initCalculLive(ListeValeurs in) throws SimuException {

        lambda0=780.*1.0e-9;
        Natomes = in.lireEntier("Natomes");
        atome = in.lireString("atome");
        sigma = in.lireDouble("sigma");
        v0 = in.lireDouble("v0");
        desaccordRelatif = in.lireDouble("domega");
        lambdal = lambda0*(1.0 - desaccordRelatif);

        VariablesAtomes = new double[2*Natomes];
        Vatomes = new double[Natomes];
        VatomesInit = new double[Natomes];
        Xatomes = new double[Natomes];
        XatomesInit = new double[Natomes];
        Pop = new Population(Natomes);

        Aff=getAffichage("distvitesse");   //Initialisation de l'affichage "distribution vitesses"

        xmin=v0-3.*sigma;
        xmax=v0+3.*sigma;

        if(xmin>0. && xmax>0.) {
            xmin=0.;
        }

        if(xmax<0. && xmin<0.) {
            xmax=0.;
        }

        debut=" "+xmin;
        fin=" "+xmax;

        Aff.xdebut = debut;
        Aff.xfin =fin;

        debut=" "+0;
        fin=" "+Natomes/2.;

        Aff.ydebut = debut;
        Aff.yfin = fin;

        VatomesInit=Pop.InitV(sigma,v0);
        XatomesInit=Pop.InitX(0.001);

        for(i=0;i<Natomes;i++){
            Vatomes[i]=VatomesInit[i];
        }
    }

    public ListeValeurs calculLive() throws SimuException {

        if( "Rubidium".equals(atome) ) {
            lambda0=780.*1.0e-9;
            tau=27e-9;
            mass=85.*1.67492728e-27;
        }

        double w0 = 2.*Math.PI*c/lambda0;
        double wl = w0*(1.0 + desaccordRelatif);

        Force approx = new Force(wl,w0,tau,mass);
        ListeValeurs out = new ListeValeurs();

        VariablesAtomes=Pop.Evolution(0.00005,Vatomes,Xatomes,approx);

        double xmin=10000.,xmax=-10000.;

        for (i=0;i<Natomes;i++){

            Vatomes[i] = VariablesAtomes[Natomes+i];
            Xatomes[i] = VariablesAtomes[i];

            if(VatomesInit[i]<xmin) xmin=VatomesInit[i];
            if(Vatomes[i]<xmin) xmin=Vatomes[i];
            if(VatomesInit[i]>xmax) xmax=VatomesInit[i];
            if(Vatomes[i]>xmax) xmax=Vatomes[i];
        }

        Histogramme=Pop.Histo(VatomesInit,Nhistos,xmin, (xmax-xmin)/Nhistos );


        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nvi",Histogramme[i]);
            out.ajouterDouble("vv",xmin + i * (xmax-xmin)/Nhistos );
        }


        Histogramme=Pop.Histo(Vatomes,Nhistos,xmin, (xmax-xmin)/Nhistos );

        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nv",Histogramme[i]);
            out.ajouterDouble("v", xmin + i * (xmax-xmin)/Nhistos );
            out.ajouterDouble("Fv",FonctionRepartition[i]);
            out.ajouterDouble("vt",1.);
            out.ajouterDouble("t",1.);
            out.ajouterDouble("ft",1.);
        }

        for (i=0;i<Natomes;i++){

            if(XatomesInit[i]<xmin) xmin=XatomesInit[i];
            if(XatomesInit[i]>xmax) xmax=XatomesInit[i];
        }

        xmin=-0.2;
        xmax=0.2;

        Histogramme=Pop.Histo(XatomesInit, Nhistos,xmin, (xmax-xmin)/Nhistos );

        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nxi",Histogramme[i]);
            out.ajouterDouble("x",xmin + i * (xmax-xmin)/Nhistos );
        }

        Histogramme=Pop.Histo(Xatomes,Nhistos,xmin, (xmax-xmin)/Nhistos );

        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nx",Histogramme[i]);
        }

        return out;
    }



    public ListeValeurs calcul(ListeValeurs in) throws SimuException {

//lecture des données utilisateur:
        Natomes = in.lireEntier("Natomes");
        sigma = in.lireDouble("sigma");
        v0 = in.lireDouble("v0");
        atome = in.lireString("atome");
        if( "Rubidium".equals(atome) ) {
            lambda0=780.*1.0e-9;
            tau=27e-9;
            mass=85.*1.67492728e-27;
        }

        double w0=2.*Math.PI*c/lambda0;
        double wl=w0*(1.0 + desaccordRelatif);

        Force approx = new Force(wl,w0,tau,mass);
        ListeValeurs out = new ListeValeurs();
        Population Pop = new Population(Natomes);
        double[] Vatomes = new double[Natomes];
        double[] Xatomes = new double[Natomes];
        double[] XatomesInit = new double[Natomes];
        double[] VatomesInit = new double[Natomes];
        double[] Histogramme = new double[Nhistos];
        double[] FonctionRepartition = new double[Nhistos];

        double xmin=10000.,xmax=-10000.;

        VatomesInit=Pop.InitV(sigma,v0);     //Initialisation de la population (Gaussienne)
        XatomesInit=Pop.InitX(0.001);

        Vatomes=Pop.Evolution(/*deltat*/1.0,VatomesInit,XatomesInit,approx); //Evolution de cette population

        Histogramme=Pop.Histo(VatomesInit,Nhistos,xmin,(xmax-xmin)/Nhistos); //Creation des histogrammes

        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nvi",Histogramme[i]);
            out.ajouterDouble("v",xmin + i * (xmax-xmin)/Nhistos );
        }

        Histogramme=Pop.Histo(Vatomes,Nhistos,xmin, (xmax-xmin)/Nhistos );

        for (i=0;i<Nhistos;i++){
            out.ajouterDouble("Nv",Histogramme[i]);
            out.ajouterDouble("Fv",FonctionRepartition[i]);
        }

        return out;
    }
}
